#!/usr/bin/perl

  require "apache-lib.pl";
  require "../guardiandigital-lib.pl";
  &ReadParseMime();




  ### Make sure we have a cert AND a key input.
  if ((($in{'certificate_up'} eq "") || ($in{'key_up'} eq ""))
      && !$in{'nokey'}) {
    &PrintError("Please enter a valid certificate AND key.");
    exit;
  }

  ### If we're using an existing key, make sure it's there.
  if ($in{'nokey'}) {
    if (!-f $in{'nokey'}) {
      &PrintError("You do not currently have a key in place.");
    }
    elsif ($in{'certificate_up'} eq "") {
      &PrintError("Please enter a valid certificate that matches the existing key.");
    }
  }

  ### ##########################################################################
  ### SSL CERTIFICATE AND KEY VALIDITY CHECK ###################################
  ### ##########################################################################
    if (!$in{'nokey'}) {
      ### Get temp files for md5 comparison.
      $cert_tmp = &tempname(); $key_tmp = &tempname();

      ### Write the certificate to a temp file.
      open CERT_TMP, ">$cert_tmp";
        print CERT_TMP $in{'certificate_up'};
      close CERT_TMP;

      ### Write the key to a temp file.
      open KEY_TMP, ">$key_tmp";
        print KEY_TMP $in{'key_up'};
      close KEY_TMP;


      ### Check the validity of the certificate.
      if (`openssl x509 -noout -modulus -in $cert_tmp 2>&1` =~ m/unable/) {
        &PrintError("You have entered an invalid certificate."); exit;
      }

      ### Check the validity of the key.
      if (`openssl rsa -noout -modulus -in $key_tmp 2>&1` =~ m/unable/) {
        &PrintError("You have entered an invalid key."); exit;
      }


      ### Get the md5 sum of the of the modulus of both the certificate and key.
      $cert_out = `openssl x509 -noout -modulus -in $cert_tmp | openssl md5`;
      $key_out = `openssl rsa -noout -modulus -in $key_tmp | openssl md5`;


      ### If the modulus (moduli?) do not match then the pair is not valid.
      if ($cert_out ne $key_out) {
        &PrintError("The certificate and key do not match each other."); exit;
      }

      ### Move the temp files into place.
      system "mv $cert_tmp $in{'certfile'}";
      system "mv $key_tmp $in{'keyfile'}";
      system "chmod 400 $in{'certfile'}";
      system "chmod 400 $in{'keyfile'}";

      $server_cert = $in{'certfile'};
      $server_key  = $in{'keyfile'};
    }

    ### Check certificate against existing key. ################################
    else {
      ### Get temp file and location of key.
      $cert_tmp = &tempname(); $keyfile = $in{'nokey'};

      ### Write the certificate to a temp file.
      open CERT_TMP, ">$cert_tmp";
        print CERT_TMP $in{'certificate_up'};
      close CERT_TMP;


      ### Check the validity of the certificate.
      if (`openssl x509 -noout -modulus -in $cert_tmp 2>&1` =~ m/unable/) {
        &PrintError("You have entered an invalid certificate."); exit;
      }

      ### Check the validity of the key.
      if (`openssl rsa -noout -modulus -in $keyfile 2>&1` =~ m/unable/) {
        &PrintError("You have entered an invalid key."); exit;
      }


      ### Get the md5 sum of the of the modulus of both the certificate and key.
      $cert_out = `openssl x509 -noout -modulus -in $cert_tmp | openssl md5`;
      $key_out = `openssl rsa -noout -modulus -in $keyfile | openssl md5`;


      ### If the modulus (moduli?) do not match then the pair is not valid.
      if ($cert_out ne $key_out) {
        &PrintError("The certificate and key do not match each other."); exit;
      }

      ### Move the new certificate into place.
      system "mv $cert_tmp $in{'certfile'}";
      system "chmod 400 $in{'certfile'}";

      $server_cert = $in{'certfile'};
      $server_key  = $in{'keyfile'};
    }
  ### ##########################################################################
  ### SSL CERTIFICATE AND KEY VALIDITY CHECK ###################################
  ### ##########################################################################


  ### ############################################## ###
  ### Save the directives with our new certificates. ###
  ### ############################################## ###

  ### Get our config.
  $conf = &get_config();
  $vconf = $conf->[$in{'virt'}];

  ### Read our file lines in.
  $lref = &read_file_lines($vconf->{'file'});

  ### Turn on SSL.
  &save_directive("SSLEngine", [ "on" ],
                  $vconf->{'members'}, $conf);

  ### Update the SSLCertificateFile directive.
  &save_directive("SSLCertificateFile", [ $server_cert ],
                  $vconf->{'members'}, $conf);

  ### Update the SSLCertificateKeyFile directive.
  &save_directive("SSLCertificateKeyFile", [ $server_key ],
                  $vconf->{'members'}, $conf);

  ### Write the file back to disk.
  &flush_file_lines();



  ### Redirect back to the server's index page.
  &redirect("virt_index.cgi?virt=$in{'virt'}&ssl=1");

