#!/usr/bin/perl

  require '../../apache-lib.pl';
  require '../../../guardiandigital-lib.pl';
  &ReadParse();


  ### ##########################################################################
  ### Setup all of our variables and complain if we don't have any #############
  ### ##########################################################################
  ### Variables for httpd.conf
  $FQDN                           = $in{'FQDN'};
  $STORENAME                      = $in{'STORENAME'};
  $ADMIN_EMAIL                    = $in{'ADMIN_EMAIL'};
  $BASEDIR                        = $in{'BASEDIR'};
  $STORE_USERNAME                 = $in{'STORE_USERNAME'};
  $STORE_GROUPNAME                = $in{'STORE_GROUPNAME'};

  ### Variables for conf/site.conf.
  $sitename                       = $in{'sitename'};
  $zipcode                        = $in{'zipcode'};
  $admin_pass                     = $in{'admin_pass'};	### MYSQL ###
  $admin_pass2                    = $in{'admin_pass2'};	### MYSQL ###
  $customer_service_email_address = $in{'customer_service_email_address'};
  $order_email_address            = $in{'order_email_address'};
  $webmaster_email_address        = $in{'webmaster_email_address'};
  $payprocess                     = $in{'payprocess'};
  $storestate                     = uc $in{'storestate'};
  $taxrate                        = $in{'taxrate'};

  ### Variables for conf/database.conf.
  $site_dbiuser                   = $in{'site_dbiuser'};
  $site_dbipwd                    = $in{'site_dbipwd'};
  $site_dbipwd2                   = $in{'site_dbipwd2'};

  ### Variables for conf/machine.conf.
  $replyto                        = $in{'replyto'};

  ### Null input checks.
  if (!$sitename) { &PrintError("Please enter a valid site name."); exit; }
  if (!$zipcode)  { &PrintError("Please enter a zip code."); exit; }
  if (!$admin_pass) {
    &PrintError("Please enter an administrative password."); exit;
  }
  if (!$admin_pass2) {
    &PrintError("Please verify your administrative password."); exit;
  }
  if (!$customer_service_email_address) {
    &PrintError("Please enter a valid e-mail address for Customer Service.");
    exit;
  }
  if (!$order_email_address) {
    &PrintError("Please enter a valid e-mail address for Orders."); exit;
  }
  if (!$webmaster_email_address) {
    &PrintError("Please enter a valid e-mail address for the Webmaster.");
    exit;
  }
  if (!$storestate) {
    &PrintError("Please enter the state this store is in."); exit;
  }
  if (!$taxrate) {
    &PrintError("Please enter the tax rate for $storestate."); exit;
  }

  if (!$site_dbiuser) {
    &PrintError("Please enter a database username."); exit;
  }
  if (!$site_dbipwd) {
    &PrintError("Please enter a database password.");
    exit;
  }
  if (!$site_dbipwd2) {
    &PrintError("Please verify your database password.");
    exit;
  }

  if (!$replyto) {
    &PrintError("Please enter a reply-to e-mail address."); exit;
  }


  ### Data formatting checks.
  if ($zipcode !~ m/^[0-9][0-9][0-9][0-9][0-9]$/) {
    &PrintError("You entered an invalid zip code.  Please renter."); exit;
  }

  if ($storestate !~ m/^[A-Za-z][A-Za-z]$/) {
    &PrintError("Please put the state into a two-letter format."); exit;
  }
  if ($taxrate !~ m/^[0-9]?[0-9]\.[0-9][0-9]$/) {
    &PrintError("Please format the tax rate as 'xx.xx'"); exit;
  }

  if ($site_dbipwd ne $site_dbipwd2) {
    &PrintError("The two database passwords are not the same.  Please re-enter"); exit;
  }

  if ($admin_pass ne $admin_pass2) {
    &PrintError("The two administrator passwords are not the same.  Please re-enter"); exit;
  }

  ### Make sure that the state exists.
  my @states = ( "al", "ak", "az", "ar", "ca", "co", "ct", "de", "dc", "fl",
                 "ga", "hi", "id", "il", "in", "ia", "ks", "ky", "la", "me",
                 "md", "ma", "mi", "mn", "ms", "mo", "mt", "ne", "nv", "nh",
                 "nj", "nm", "ny", "nc", "nd", "oh", "ok", "or", "pa", "ri",
                 "sc", "sd", "tn", "tx", "ut", "vt", "va", "wa", "wv", "wi",
                 "wy" );

  my @mm = grep {/$storestate/i} @states;

  if (!@mm) { &PrintError("'$storestate' is an invalid state."); exit; }


  ##############################################################################
  ### Make sure user does not exist.
  ##############################################################################
    ### Get the root password.
    open FILE, "/etc/webtool/SystemManager/config";
      while (<FILE>) { if ($_ =~ m/^pass\=(.*)$/) { $dbi_pass = $1; } }
    close FILE;

    ### Make the connection to the DBMS on localhost.
    use DBI;
    $dbh = DBI->connect("DBI:mysql:host=localhost:database=$STORENAME",
                        "root", "$dbi_pass");

    if ($dbh) { print; }
  ##############################################################################
  ### Make sure database, user does not exist.
  ##############################################################################


  ### Good ol' header.
  &header("Setup a Store:  Step 3 of 3", "");

  $sslinfo_help   = &hlink("[ Certificate Information ]", "SSLInfo");
  $ssl_method_0   = &hlink("Do Not Use SSL", "SSLInfo");
  $ssl_method_1   = &hlink("Upload Certificate and Key", "SSLInfo");
  $ssl_method_2   = &hlink("Generate Certificate and Key", "SSLInfo");
  $sslcert_help   = &hlink("SSL Certificate", "SSLCert");
  $sslkey_help    = &hlink("SSL Key", "SSLKey");
  $postmaster_ucb = &user_chooser_button("maildomain_defrec", 0, 0);

  $authority_str    = &hlink("Authority Name", "ssl_authority");
  $email_str        = &hlink("E-Mail Address", "ssl_email");
  $department_str   = &hlink("Department", "ssl_department");
  $organization_str = &hlink("Organization", "ssl_organization");
  $city_str         = &hlink("City", "ssl_city");
  $state_str        = &hlink("State or Province", "ssl_state");
  $country_str      = &hlink("Country", "ssl_country");



print <<EOF;
  <FORM ACTION="verify_store.cgi" ENCTYPE="multipart/form-data" METHOD="POST">
  <INPUT TYPE="HIDDEN" NAME="preview" VALUE="yes">
  <INPUT TYPE="HIDDEN" NAME="FQDN" VALUE="$FQDN">
  <INPUT TYPE="HIDDEN" NAME="STORENAME" VALUE="$STORENAME">
  <INPUT TYPE="HIDDEN" NAME="ADMIN_EMAIL" VALUE="$ADMIN_EMAIL">
  <INPUT TYPE="HIDDEN" NAME="STORE_USERNAME" VALUE="$STORE_USERNAME">
  <INPUT TYPE="HIDDEN" NAME="STORE_GROUPNAME" VALUE="$STORE_GROUPNAME">
  <INPUT TYPE="HIDDEN" NAME="sitename" VALUE="$sitename">
  <INPUT TYPE="HIDDEN" NAME="zipcode" VALUE="$zipcode">
  <INPUT TYPE="HIDDEN" NAME="admin_pass" VALUE="$admin_pass">
  <INPUT TYPE="HIDDEN" NAME="customer_service_email_address" VALUE="$customer_service_email_address">
  <INPUT TYPE="HIDDEN" NAME="order_email_address" VALUE="$order_email_address">
  <INPUT TYPE="HIDDEN" NAME="webmaster_email_address" VALUE="$webmaster_email_address">
  <INPUT TYPE="HIDDEN" NAME="payprocess" VALUE="$payprocess">
  <INPUT TYPE="HIDDEN" NAME="storestate" VALUE="$storestate">
  <INPUT TYPE="HIDDEN" NAME="taxrate" VALUE="$taxrate">
  <INPUT TYPE="HIDDEN" NAME="site_dbiuser" VALUE="$site_dbiuser">
  <INPUT TYPE="HIDDEN" NAME="site_dbipwd" VALUE="$site_dbipwd">
  <INPUT TYPE="HIDDEN" NAME="replyto" VALUE="$replyto">
  <INPUT TYPE="HIDDEN" NAME="BASEDIR" VALUE="/home/httpd">

  <CENTER><TABLE BORDER="0" WIDTH="620" CELLPADDING="0" CELLSPACING=0>
  <TR $tb><TD COLSPAN=2><DIV CLASS=ttitle><B>Setup a Store</B></DIV></TD></TR>
  <TR $cb><TD COLSPAN=2>&nbsp;</TD></TR>
  <TR $cb><TD COLSPAN=2><DIV CLASS=nbody>
    Below is the final step of setting up a new store.

    <UL>
      <LI><B>SSL Certificate</B> is the certificate you wish to use for this store.</LI>
      <LI><B>SSL Certificate</B> is the key you wish to use for this store.</LI>
    </UL>

    <B>$sslinfo_help</B>

    <P><B>Clicking on "Preview Store" will bring you to a confirmation screen.</B>
  </TD></TR>

 <TR $cb><TD COLSPAN=2>&nbsp;</TD></TR>
  <TR $cb>
    <TD WIDTH=20>&nbsp;</TD>
    <TD>
      <TABLE BORDER=0 WIDTH=100% CELLSPACING=0 CELLPADDING=0>

      <TR $tb>
        <TD COLSPAN="2"><DIV CLASS="thead"> <B>SSL Options</B> </TD>
      </TR>

      <TR $cb><TD COLSPAN=2>&nbsp;</TD></TR>
      <TR $cb>
        <TD COLSPAN=2><DIV CLASS=nbody><B>
          <INPUT TYPE=RADIO NAME=sslmethod VALUE=0> $ssl_method_0
          <BR><INPUT TYPE=RADIO NAME=sslmethod VALUE=1> $ssl_method_1
          <BR><INPUT TYPE=RADIO NAME=sslmethod VALUE=2 CHECKED> $ssl_method_2

        </B></DIV></TD>
      </TR>
      <TR $cb><TD COLSPAN=2>&nbsp;</TD></TR>

      <TR $tb>
        <TD COLSPAN="2"><DIV CLASS="thead">
          <B>Upload SSL Certificate and Key</B>
        </TD>
      </TR>

      <TR $cb>
        <TD><DIV CLASS="nbody"><B>$sslcert_help</B></DIV></TD>
        <TD><INPUT TYPE="file" NAME="certfile" SIZE=30></TD>
      </TR>

      <TR $cb>
        <TD><DIV CLASS="nbody"><B>$sslkey_help</B></DIV></TD>
        <TD><INPUT TYPE="file" NAME="keyfile" SIZE=30></TD>
      </TR>

      <TR $cb><TD COLSPAN=2>&nbsp;</TD></TR>

      <TR $tb>
        <TD COLSPAN="2"><DIV CLASS="thead">
          <B>Generate New SSL Certificate and Key</B>
        </TD>
      </TR>

      <TR $cb>
        <td><DIV CLASS=nbody><b>$authority_str</b></td>
        <td><INPUT NAME=commonName SIZE=30 MAXLENGTH=64 VALUE=$FQDN></td>
      </TR>

      <TR $cb>
        <td><DIV CLASS=nbody><b>$organization_str</b></td>
        <td><INPUT NAME=organizationName SIZE=30 VALUE="$sitename"></td>
      </TR>

      <TR $cb>
        <td><DIV CLASS=nbody><b>$department_str</b></td>
        <td><INPUT NAME=organizationalUnitName SIZE=30></td>
      </TR>

      <TR $cb>
        <td><DIV CLASS=nbody><b>$email_str</b></td>
        <td><INPUT NAME=emailAddress SIZE=30 MAXLENGTH=40></td>
      </TR>

      <TR $cb>
        <td><DIV CLASS=nbody><b>$city_str</b></td>
        <td><INPUT NAME=localityName SIZE=30></td>
      </TR>

      <TR $cb>
        <td><DIV CLASS=nbody><b>$state_str</b></td>
        <td><INPUT NAME=stateOrProvinceName SIZE=15></td>
      </TR>

      <TR $cb>
        <td><DIV CLASS=nbody><b>$country_str</b></td>
        <td><INPUT NAME=countryName SIZE=3 MAXLENGTH=2></td>
      </TR>


      <TR $cb>
        <TD COLSPAN="2" ALIGN="RIGHT"><DIV CLASS=nbody>
          <INPUT TYPE="SUBMIT" VALUE="Preview Store">
        </TD>
      </TR>
      </TABLE>
    </TD>
  </TR>
  </TABLE></CENTER></FORM><P>
EOF

  ### Good ol' footer.
  &ryanFooter("/VHost/opensales");
