#!/usr/bin/perl

  require '../../apache-lib.pl';
  require '../../../guardiandigital-lib.pl';
  &ReadParseMime();

  $openssl = "/usr/bin/openssl";

  ### Variables for httpd.conf
  $HTTPD_CONF = "/etc/httpd/conf/httpd.conf";
  $FQDN                          = $in{'FQDN'};
  $STORENAME                     = $in{'STORENAME'};
  $ADMIN_EMAIL                   = $in{'ADMIN_EMAIL'};
  $BASEDIR                       = $in{'BASEDIR'};
  $STORE_USERNAME                 = $in{'STORE_USERNAME'};
  $STORE_GROUPNAME                = $in{'STORE_GROUPNAME'};

  ### Variables for conf/site.conf.
  $sitename                       = $in{'sitename'};
  $zipcode                        = $in{'zipcode'};
  $admin_pass                     = $in{'admin_pass'};
  $customer_service_email_address = $in{'customer_service_email_address'};
  $order_email_address            = $in{'order_email_address'};
  $webmaster_email_address        = $in{'webmaster_email_address'};
  $payprocess                     = $in{'payprocess'};
  $storestate                     = uc $in{'storestate'};
  $taxrate                        = $in{'taxrate'};

  ### Variables for conf/database.conf.
  $site_dbiuser                   = $in{'site_dbiuser'};
  $site_dbipwd                    = $in{'site_dbipwd'};

  ### Variables for conf/machine.conf.
  $replyto                        = $in{'replyto'};

  ### Hash key.
  $hash_key                       = `head -50 /dev/urandom | openssl md5`;
  chomp $hash_key;

  $sslmethod                      = $in{'sslmethod'};

    ### Method 1:  Upload certificate and key.
    $ssl_certificate                = $in{'certfile'};
    $ssl_key                        = $in{'keyfile'};

    ### Method 2:  Generate new certificate and key.
    $gen_commonName                 = $in{'commonName'};
    $gen_organizationName           = $in{'organizationName'};
    $gen_organizationalUnitName     = $in{'organizationalUnitName'};
    $gen_emailAddress               = $in{'emailAddress'};
    $gen_localityName               = $in{'localityName'};
    $gen_stateOrProvinceName        = $in{'stateOrProvinceName'};
    $gen_countryName                = $in{'countryName'};




  ### ##########################################################################
  ### Do one last sanity check on the variables in case they were changed ######
  ### on the verification page. ################################################
  ### ##########################################################################

  ######################
  ### APACHE OPTIONS ###
  ######################
    ### Null input checks.
    if (!$FQDN) { &PrintError("Please enter a valid domain name."); exit; }
    if (!$STORENAME) { &PrintError("Please enter a store name."); exit; }
    if (!$ADMIN_EMAIL) {
      &PrintError("Please enter the administrative e-mail address."); exit;
    }

    ### Data formatting checks.
    if ($FQDN !~ m/^[A-Za-z0-9\-\.]+\.[A-Za-z]+$/) {
      &PrintError("'$FQDN' is not a valid fully-qualified domain name.");
    }
    if ($STORENAME =~ m/\ /) {
      &PrintError("The Store Name cannot have any spaces in it."); exit;
    }
    if ($ADMIN_EMAIL !~ m/^.*@.*$/) {
      &PrintError("'$ADMIN_EMAIL' is an invalid e-mail address.'");
    }

  ### ################### ###
  ### ALLCOMMERCE OPTIONS ###
  ### ################### ###
    ### Null input checks.
    if (!$sitename) { &PrintError("Please enter a valid site name."); exit; }
    if (!$zipcode)  { &PrintError("Please enter a zip code."); exit; }
    if (!$customer_service_email_address) {
      &PrintError("Please enter a valid e-mail address for Customer Service.");
      exit;
    }
    if (!$order_email_address) {
      &PrintError("Please enter a valid e-mail address for Orders."); exit;
    }
    if (!$webmaster_email_address) {
      &PrintError("Please enter a valid e-mail address for the Webmaster.");
      exit;
    }
    if (!$storestate) {
      &PrintError("Please enter the state this store is in."); exit;
    }
    if (!$taxrate) {
      &PrintError("Please enter the tax rate for $storestate."); exit;
    }

    if (!$site_dbiuser) {
      &PrintError("Please enter a database username."); exit;
    }
    if (!$site_dbipwd) {
      &PrintError("Please enter a password for username '$site_dbiuser'");
      exit;
    }
    if (!$replyto) {
      &PrintError("Please enter a reply-to e-mail address."); exit;
    }


    ### Data formatting checks.
    if ($zipcode !~ m/^[0-9][0-9][0-9][0-9][0-9]$/) {
      &PrintError("You entered an invalid zip code.  Please renter."); exit;
    }

    if ($storestate !~ m/^[A-Za-z][A-Za-z]$/) {
      &PrintError("Please put the state into a two-letter format."); exit;
    }
    if ($taxrate !~ m/^[0-9]?[0-9]\.[0-9][0-9]$/) {
      &PrintError("Please format the tax rate as 'xx.xx'"); exit;
    }

    ### Make sure that the state exists.
    my @states = ( "al", "ak", "az", "ar", "ca", "co", "ct", "de", "dc", "fl",
                   "ga", "hi", "id", "il", "in", "ia", "ks", "ky", "la", "me",
                   "md", "ma", "mi", "mn", "ms", "mo", "mt", "ne", "nv", "nh",
                   "nj", "nm", "ny", "nc", "nd", "oh", "ok", "or", "pa", "ri",
                   "sc", "sd", "tn", "tx", "ut", "vt", "va", "wa", "wv", "wi",
                   "wy" );

    my @mm = grep {/$storestate/i} @states;

    if (!@mm) { &PrintError("'$storestate' is an invalid state."); exit; }


  ### ########### ###
  ### SSL OPTIONS ###
  ### ########### ###

    ### ########################################################################
    ### METHOD 0:  We do not want to use SSL.
    ### ########################################################################
    if ($sslmethod == 0) {
        $SSL_STATUS      = "<FONT COLOR=RED>SSL Will Be Disabled</FONT>";
        $ssl_certificate = "";
        $ssl_key         = "";
    }

    ### ########################################################################
    ### METHOD 1:  We want to upload a certificate and key.
    ### ########################################################################
    elsif ($sslmethod == 1) {

      ### Must enter BOTH key and a certificiate.
      if (($ssl_certificate && !$ssl_key) ||
          (!$ssl_certificate && $ssl_key) ||
          (!$ssl_certificate && !$ssl_key)) {
        &PrintError("You must enter both a valid certificate and key.");
      }

      ### If both are entered, make sure they match up. 
      if ($ssl_certificate && $ssl_key) {

        ### Get temp files for md5 comparison.
        $cert_tmp = &tempname(); $key_tmp = &tempname();

        ### Write the certificate to a temp file.
        open CERT_TMP, ">$cert_tmp";
          print CERT_TMP $ssl_certificate;
        close CERT_TMP;

        ### Write the key to a temp file.
        open KEY_TMP, ">$key_tmp";
          print KEY_TMP $ssl_key;
        close KEY_TMP;


        ### Check the validity of the certificiate.
        if (`openssl x509 -noout -modulus -in $cert_tmp 2>&1` =~ m/unable/) { 
          &PrintError("You have entered an invalid certificate.");
        }

        ### Check the validity of the key.
        if (`openssl rsa -noout -modulus -in $key_tmp 2>&1` =~ m/unable/) { 
          &PrintError("You have entered an invalid key.");
        }


        ### Get the md5 sum of the of the modulus of both certificate and key.
        $cert_out = `openssl x509 -noout -modulus -in $cert_tmp | openssl md5`;
        $key_out = `openssl rsa -noout -modulus -in $key_tmp | openssl md5`;

        ### If the modulus (moduli?) do not match then the pair is not valid.
        if ($cert_out ne $key_out) {
          &PrintError("The certificate and key do not match each other.");
        }

        ### Delete our temp files.
        unlink $cert_tmp, $key_tmp;

        ### Set our pretty little status variables.
        $SSL_STATUS      = "<FONT COLOR=GREEN>SSL Will Be Enabled</FONT>";
      }
    }

    ### ########################################################################
    ### METHOD 2:  We want to generate a certificate and key.
    ### ########################################################################
    elsif ($sslmethod == 2) {

      ### Verify that all our items are input.
      if (!$gen_commonName) {
        &PrintError("Please enter an Authority Name");
      }
      elsif (!$gen_organizationName) {
        &PrintError("Please enter an Organization");
      }
      elsif (!$gen_emailAddress) {
        &PrintError("Please enter an E-Mail Address");
      }
      elsif (!$gen_localityName) {
        &PrintError("Please enter a City");
      }
      elsif (!$gen_stateOrProvinceName) {
        &PrintError("Please enter a State or Provence");
      }
      elsif (!$gen_countryName) { &PrintError("Please enter a Country Name"); }

      ### Set our pretty little status variables.
      $SSL_STATUS      = "<FONT COLOR=GREEN>SSL Will Be Enabled</FONT>";
    }

  ### ############################### ###
  ### We want to preview our entries. ###
  ### ############################### ###
  if ($in{'preview'}) {

    ### Get all of the user names.
    $users_str  = "<SELECT NAME=\"STORE_USERNAME\">\n";
    $users_str .= "  <OPTION>Please Select</OPTION>\n";
      setpwent();
        while (@tmp = getpwent()) {
          if ($tmp[2] >= 500) {
            if ($tmp[0] eq $STORE_USERNAME) {
              $users_str .= "  <OPTION SELECTED>$tmp[0]</OPTION>\n";
            } else {
              $users_str .= "  <OPTION>$tmp[0]</OPTION>\n";
            }
          }
        }
      endpwent();
    $users_str .= "</SELECT>\n";

    ### Get all of the group names.
    $groups_str  = "<SELECT NAME=\"STORE_GROUPNAME\">\n";
    $groups_str .= "  <OPTION>Please Select</OPTION>\n";
      setgrent();
        while (@tmp = getgrent()) {
          if ($tmp[2] >= 100) {
            if ($tmp[0] eq $STORE_GROUPNAME) {
              $groups_str .= "  <OPTION SELECTED>$tmp[0]</OPTION>\n";
            }
            else {
              $groups_str .= "  <OPTION>$tmp[0]</OPTION>\n";
            }
          }
        }
      endgrent();
    $groups_str .= "</SELECT>\n";


    ### Good ol' header.
    &header("Steup a Store:  Verification", "");

    print <<EOF01;
<FORM ACTION="verify_store.cgi" METHOD="POST" ENCTYPE="multipart/form-data">
<INPUT TYPE="HIDDEN" NAME="BASEDIR" VALUE="$BASEDIR">
<INPUT TYPE="HIDDEN" NAME="admin_pass" VALUE="$admin_pass">
<INPUT TYPE="HIDDEN" NAME="site_dbiuser" VALUE="$site_dbiuser">
<INPUT TYPE="HIDDEN" NAME="site_dbipwd" VALUE="$site_dbipwd">
<INPUT TYPE="HIDDEN" NAME="hash_key" VALUE="$hash_key">

<CENTER><TABLE BORDER=0 CELLPADDING=0 CELLSPACING=0 WIDTH=620>
  <TR $tb><TD COLSPAN=2><DIV CLASS=ttitle><B>Setup a Store</B></DIV></TD></TR>
  <TR $cb><TD COLSPAN=2>&nbsp;</TD></TR>
  <TR $cb><TD COLSPAN=2><DIV CLASS=nbody> 
    Below you can confirm your settings.

    <P><B>Please verify that all of the below information is correct. 
          Incorrect values will cause your store to malfunction, or not work at
          all.

    <P><B>Clicking on "Create Store" will complete the setup process.</B>

  </TD></TR>

 <TR $cb><TD COLSPAN=2>&nbsp;</TD></TR>
  <TR $cb>
    <TD WIDTH=20>&nbsp;</TD>
    <TD>
      <TABLE BORDER=0 WIDTH=100% CELLSPACING=0 CELLPADDING=0>
      <!-- ####################### --> 
      <!-- Webserver Configuration -->
      <!-- ####################### --> 
      <TR $tb><TD COLSPAN="2">
        <DIV CLASS="thead"><B>Webserver Configuration</B></DIV>
      </TD></TR>

      <TR $cb>
        <TD><DIV CLASS=nbody><B>Fully-Qualified Domain Name</TD>
        <TD><INPUT TYPE=TEXT NAME=FQDN SIZE=30 VALUE="$FQDN"></TD>
      </TR>

      <TR $cb>
        <TD><DIV CLASS=nbody><B>Storename Indentifier</TD>
        <TD><INPUT TYPE=TEXT NAME=STORENAME SIZE=30 VALUE="$STORENAME"></TD>
      </TR>

      <TR $cb>
        <TD><DIV CLASS=nbody><B>Administrator E-Mail Address</TD>
        <TD><INPUT TYPE=TEXT NAME=ADMIN_EMAIL SIZE=30 VALUE="$ADMIN_EMAIL"></TD>
      </TR>

      <TR $cb>
        <TD><DIV CLASS="nbody"><B>Store User Name</B></DIV></TD>
        <TD><DIV CLASS="toolfont">$users_str</DIV></TD>
      </TR>

      <TR $cb>
        <TD><DIV CLASS="nbody"><B>Store Group Name</B></DIV></TD>
        <TD><DIV CLASS="toolfont">$groups_str</DIV></TD>
      </TR>

      <TR $cb> <TD COLSPAN="2"> &nbsp; </TD> </TR>
      <TR $cb> <TD COLSPAN="2"> &nbsp; </TD> </TR>

      <!-- ######################### --> 
      <!-- AllCommerce Configuration -->
      <!-- ######################### --> 
      <TR $tb><TD COLSPAN="2">
        <DIV CLASS="thead"><B>AllCommerce Configuration</B></DIV>
      </TD></TR>

      <TR $cb>
        <TD><DIV CLASS=nbody><B>Site Name</TD>
        <TD><INPUT TYPE=TEXT NAME=sitename SIZE=30 VALUE="$sitename"></TD>
      </TR>

      <TR $cb>
        <TD><DIV CLASS=nbody><B>Store Zip Code</TD>
        <TD><INPUT TYPE=TEXT NAME=zipcode SIZE=5 MAXLENGTH="5" VALUE="$zipcode"></TD>
      </TR>

      <TR $cb>
        <TD><DIV CLASS=nbody><B>Customer Service E-Mail</TD>
        <TD><INPUT TYPE=TEXT NAME=customer_service_email_address SIZE=30 VALUE="$customer_service_email_address"></TD>
      </TR>

      <TR $cb>
        <TD><DIV CLASS=nbody><B>Order E-Mail</TD>
        <TD><INPUT TYPE=TEXT NAME=order_email_address SIZE=30 VALUE="$order_email_address"></TD>
      </TR>

      <TR $cb>
        <TD><DIV CLASS=nbody><B>Webmaster E-Mail</TD>
        <TD><INPUT TYPE=TEXT NAME=webmaster_email_address SIZE=30 VALUE="$webmaster_email_address"></TD>
      </TR>

      <TR $cb>
        <TD><DIV CLASS=nbody><B>Pay Process</TD>
        <TD><INPUT TYPE=TEXT NAME=payprocess SIZE=30 VALUE="$payprocess"></TD>
      </TR>

      <TR $cb>
        <TD><DIV CLASS=nbody><B>Store State</TD>\
        <TD><INPUT TYPE=TEXT NAME=storestate SIZE=2 VALUE="$storestate" MAXLENGTH=2></TD>
      </TR>

      <TR $cb>
        <TD><DIV CLASS=nbody><B>Tax Rate</TD>
        <TD><INPUT TYPE=TEXT NAME=taxrate SIZE=5 VALUE="$taxrate" MAXLENGTH=5>%</TD>
      </TR>

      <TR $cb>
        <TD><DIV CLASS=nbody><B>Reply To E-Mail</TD>
        <TD><INPUT TYPE=TEXT NAME=replyto SIZE=30 VALUE="$replyto"></TD>
      </TR>

      <TR $cb> <TD COLSPAN="2"> &nbsp; </TD> </TR>
      <TR $cb> <TD COLSPAN="2"> &nbsp; </TD> </TR>


      <!-- ################# --> 
      <!-- SSL Configuration -->
      <!-- ################# --> 
      <TR $tb><TD COLSPAN="2">
        <DIV CLASS="thead"><B>SSL Configuration</B></DIV>
      </TD></TR>

      <TR $cb> <TD COLSPAN="2"> &nbsp; </TD> </TR>

      <TR $cb>
        <TD><DIV CLASS=nbody><B>SSL Status</B></TD>
        <TD><DIV CLASS=nbody><B>$SSL_STATUS</B>
          <INPUT TYPE=HIDDEN NAME=sslmethod VALUE="$sslmethod">

          <INPUT TYPE=HIDDEN NAME=certfile VALUE="$ssl_certificate">
          <INPUT TYPE=HIDDEN NAME=keyfile VALUE="$ssl_key">

          <INPUT TYPE=HIDDEN NAME=commonName VALUE="$gen_commonName">
          <INPUT TYPE=HIDDEN NAME=organizationName VALUE="$gen_organizationName">
          <INPUT TYPE=HIDDEN NAME=organizationalUnitName VALUE="$gen_organizationalUnitName">
          <INPUT TYPE=HIDDEN NAME=emailAddress VALUE="$gen_emailAddress">
          <INPUT TYPE=HIDDEN NAME=localityName VALUE="$gen_localityName">
          <INPUT TYPE=HIDDEN NAME=stateOrProvinceName VALUE="$gen_stateOrProvinceName">
          <INPUT TYPE=HIDDEN NAME=countryName VALUE="$gen_countryName">
        </TD>
      </TR>

      <TR $cb> <TD COLSPAN="2"> &nbsp; </TD> </TR>

      <TR $cb>
        <TD COLSPAN=2 ALIGN=RIGHT><DIV CLASS=nbody>
          <INPUT TYPE=SUBMIT VALUE="Create Store">
          <INPUT TYPE=SUBMIT NAME="preview" VALUE="Preview Changes">
        </TD>
      </TR>

    </TABLE>
  </TD>
</TR>
</TABLE></CENTER></FORM>
EOF01

    &ryanFooter($ENV{"HTTP_REFERER"});
    exit;

  }

  ### ############################## ###
  ### Go ahead and create the store. ###
  ### ############################## ###
  else {

    ### Get the root password.
    open FILE, "/etc/webtool/SystemManager/config";
      while (<FILE>) { if ($_ =~ m/^pass\=(.*)$/) { $dbi_pass = $1; } }
    close FILE;

    ### Make the connection to the DBMS on localhost.
    use DBI;
    $dbh = DBI->connect("DBI:mysql:host=localhost", "root", "$dbi_pass");

    ### Create the database.
    $rv = $dbh->do("CREATE DATABASE $STORENAME");

      ### Verify that database does not exist, if it does then tell user.
      if ($rv != 1) {
        &PrintError("Database '$STORENAME' already exists.  Please choose a new Storename Identifier");
        exit;
      }

    ### Create the username.
    $rv = $dbh->do("INSERT INTO mysql.user VALUES ('localhost', '$site_dbiuser',
                    PASSWORD('$site_dbipwd'), 'N', 'N', 'N', 'N', 'N', 'N', 'N',
                    'N', 'N', 'N', 'N', 'N', 'N', 'N')");

    ### Setup grant tables so they can only access their database.
    $rv = $dbh->do("INSERT INTO mysql.db VALUES ('localhost', '$STORENAME',
                   '$site_dbiuser', 'Y', 'Y', 'Y', 'Y', 'Y', 'Y', 'Y', 'Y', 'Y',
                   'Y')");

    ### Reload grant tables.
    $rv = $dbh->do("FLUSH PRIVILEGES");


    ### Move into the $BASEDIR for all of our operations.
    chdir $BASEDIR;

    ### Copy the new-store into place.
    system "/bin/cp -a new-store $STORENAME";

    ### Read the skeleton file into tables_conf.
    open TABLES, "$STORENAME/conf/tables.mysql.skel"; $i = 0;
    while (<TABLES>) { chomp $_;
      if ($_ !~ /^#/) { if ($_ eq ");") { $tables_conf[$i++] .= $_; }
      else { $tables_conf[$i] .= $_; } }
    }
    close TABLES;

    ### Read the administrator entries into admin.
    open ADMIN, "$STORENAME/conf/admin.mysql"; $i = 0;
    while (<ADMIN>) { chomp $_;
      if ($_ !~ /^#/) { if ($_ eq ");") { $admin[$i++] .= $_; }
      else { $admin[$i] .= $_; } }
    }
    close ADMIN;

    ### Read the shipping table into into shipping_things.
    open SHIPPING, "$STORENAME/tools/data/shipping_things_table.dbid"; $i = 0;
    while (<SHIPPING>) { chomp $_;
      if ($_ !~ /^#/) { $shipping_things[$i++] = $_; }
    }
    close SHIPPING;

    ### Read the zipmap table into into zipmap.
    open ZIPMAP, "$STORENAME/tools/data/guardiandigital_zipmap.sql"; $i = 0;
    while (<ZIPMAP>) { chomp $_;
      if ($_ !~ /^#/) { $zipmap[$i++] = $_; }
    }
    close ZIPMAP;

    ### We're done with the DBMS (for now) so let's disconnect.
    $dbh->disconnect();

    ### ...and then reconnect to the new database as our new user.
    $dbh = DBI->connect("DBI:mysql:database=$STORENAME;;host=$dbi_host",
                        "$site_dbiuser", "$site_dbipwd");

    ### Setup the table structure for the new database.
    foreach $tc (@tables_conf) { $rv = $dbh->do("$tc"); }
    foreach $a (@admin)        { $rv = $dbh->do("$a"); }

    ### Setup the shipping table and zipmap information.
    foreach $st (@shipping_things) { $rv = $dbh->do("$st"); }
    foreach $zm (@zipmap)          { $rv = $dbh->do("$zm"); }

    ### Get the first three digits of our zip code for shipping tables.
    ($ups_code) = ($zipcode =~ m/^([0-9][0-9][0-9])[0-9][0-9]$/);

      ### Go into the tools directory...
      chdir "$STORENAME/tools";

##### BEGIN DEVEL #####
      $sth = $dbh->prepare("SELECT Filename FROM zipmap
                              WHERE Start <= $ups_code AND End >= $ups_code");
      $sth->execute(); ($filename) = $sth->fetchrow_array(); $sth->finish();
##### END DEVEL #####

      ### ...move the CSV for our zone into place...
      system "mv data/UPS_ZONES/$filename data/";

      ### ...and run 'import_shipping_tables.pl'.
      system "./import_shipping_tables.pl";

      ### Open the rates file and insert it into the database.
      open RATES, "data/UPS.rates.dbid";
        while (<RATES>) { $_ =~ s/\\g//; $rv = $dbh->do("$_"); }
      close RATES;

      ($zone) = ($filename =~ m/^([0-9]+)\.csv$/);

      ### Open the zones file and insert it into the database.
      open ZONES, "data/UPS.zones.$zone.dbid";
        while (<ZONES>) { $_ =~ s/\\g//; $rv = $dbh->do("$_"); }
      close ZONES;

    ### And finally, set the administrators password.
    $rv = $dbh->do("UPDATE contacts SET passwd = '$admin_pass' WHERE uname = 'admin'");



    ### All done so go back to the base directory.
    chdir $BASEDIR;

    ### Reload grant tables.
    $rv = $dbh->do("FLUSH PRIVILEGES");


    ### ########################################################################
    ### And finally, let's write all the configuration files ###################
    ### ########################################################################

    ### Make the SSL directory, and chmod it accordingly.
    system "mkdir -p $BASEDIR/$STORENAME/ssl";
    system "chmod 500 $BASEDIR/$STORENAME/ssl";

      ### ######################################################################
      ### SSL KEY MAINTENANCE ##################################################
      ### ######################################################################

      ### Save our uploaded cert/key to the appropriate files.
      if ($sslmethod == 1) {
        $certfile = "/home/httpd/$STORENAME/ssl/certificate.pem";
        $keyfile  = "/home/httpd/$STORENAME/ssl/key.pem";

        ### Write the certificate to a file.
        open CERTIFICATE, ">$certfile";
          print CERTIFICATE $ssl_certificate;
        close CERTIFICATE;

        ### Write the key to a file.
        open KEY, ">$keyfile";
          print KEY $ssl_key;
        close KEY;

        ### Sanitize modes and permissions.
        chmod 0400, $certfile, $keyfile;
        system "/bin/chown root.root $certfile $keyfile";

        ### Activate SSL.
        $SSLONOFF = "SSLEngine on";
      }

      ### Generate our cert/key.
      elsif ($sslmethod == 2) {
        $certfile = "/home/httpd/$STORENAME/ssl/certificate.pem";
        $keyfile  = "/home/httpd/$STORENAME/ssl/key.pem";

        ### Open a pipe to ssl and give it our info, and output cert and key.
        open CA, "| openssl req -new -x509 -days 365 -nodes -out $certfile -keyout $keyfile -config /etc/ssl/openssl.cnf > /dev/null 2>&1";
          print CA "$gen_countryName\n";
          print CA "$gen_stateOrProvinceName\n";
          print CA "$gen_localityName\n";
          print CA "$gen_organizationName\n";
          print CA "$gen_organizationalUnitName\n";
          print CA "$gen_commonName\n";
          print CA "$gen_emailAddress\n";
        close CA;

        ### Sanitize modes and permissions.
        chmod 0400, $certfile, $keyfile;
        system "/bin/chown root.root $certfile $keyfile";

        ### Activate SSL.
        $SSLONOFF = "SSLEngine on";

      }
      else { $SSLONOFF = "SSLEngine off"; }


    ### Write /etc/httpd/conf/httpd.conf
    &obtain_lock("$HTTPD_CONF");
    open VHOST, ">>$HTTPD_CONF";

    print VHOST <<EOF02;
<VirtualHost $FQDN:80>
  ### General options.
  ServerAdmin $ADMIN_EMAIL
  DocumentRoot $BASEDIR/$STORENAME
  ServerName $FQDN
  ErrorLog $BASEDIR/$STORENAME/logs/error_log
  TransferLog $BASEDIR/$STORENAME/logs/access_log
  ScriptAlias /cgi-bin/ $BASEDIR/$STORENAME/cgi-bin/

  ### We want to use 'index.cgi' as a directory index.
  DirectoryIndex index.cgi

  ### Setup special file handlers.
  AddHandler cgi-script .cgi
  AddType text/html .shtml
  AddHandler server-parsed .shtml

  ### Define options for our directory.
  <Directory $BASEDIR/$STORENAME/>
    Options Includes ExecCGI FollowSymLinks
  </Directory>

  ### Only access managment screens from SSL.
  <Directory $BASEDIR/$STORENAME/cgi-bin/om/admin>
    Deny from all
  </Directory>
</VirtualHost>

<VirtualHost $FQDN:443>
  ### General options.
  ServerAdmin $ADMIN_EMAIL
  DocumentRoot $BASEDIR/$STORENAME
  ServerName $FQDN
  ErrorLog $BASEDIR/$STORENAME/logs/ssl-error_log
  TransferLog $BASEDIR/$STORENAME/logs/ssl-access_log
  ScriptAlias /cgi-bin/ $BASEDIR/$STORENAME/cgi-bin/

  ### Setup SSL.
  $SSLONOFF
EOF02

  if ($SSLONOFF eq "SSLEngine on") {
    print VHOST "  SSLCertificateFile $BASEDIR/$STORENAME/ssl/certificate.pem\n";
    print VHOST "  SSLCertificateKeyFile $BASEDIR/$STORENAME/ssl/key.pem\n";
  }

    print VHOST <<EOF03;

  ### Define options for our directory.
  <Directory $BASEDIR/$STORENAME/>
    Options Includes ExecCGI FollowSymLinks
  </Directory>

   ### Special directives for /cgi-bin/.
   <Directory "$BASEDIR/$STORENAME/cgi-bin">
       AllowOverride all
       Options Includes FollowSymlinks ExecCGI
       Allow from all
       SSLOptions +StdEnvVars
   </Directory>

  ### Setup the standard $ENV{'.*'} variables.
  <Files ~ "\.(cgi|shtml)$">
    SSLOptions +StdEnvVars
  </Files>
</VirtualHost>

EOF03
    close VHOST;
    &release_lock("$HTTPD_CONF");


    ### ########################################################################
    ### Generate conf/merchant_conf. ###########################################
    ### ########################################################################
    &obtain_lock("$STORENAME/conf/merchant_conf");
    open MERCHANT, ">$STORENAME/conf/merchant_conf";

    print MERCHANT <<MERCHANT_EOF;
MCK_TYPE =

CCPS_SSL_HOST = https://cr.cybercash.com/cgi-bin/
CCPS_HOST = http://cr.cybercash.com/cgi-bin/
CCW_HOST = a.merchant.agilewallet.com

CYBERCASH_ID = test-mck
HASH_SECRET  = secret-test-mck
BLOWFISH_KEY = This_is_only_a_temp_key
MERCHANT_KEY = gmRJp8rbzzcRdI5g8eLRScFeeQSWew5FsBzq6bxJDSJIiWhq7f

TEMPLATE_DIR = $BASEDIR/$STORENAME/bits/eng/html/mck/
ORDER_DB = $BASEDIR/$STORENAME/logs/cybercash_order.log
NOTIFICATION_LOG = $BASEDIR/$STORENAME/logs/cybercash_notification.log

MIME_TYPE_FILE = /home/httpd/os_allcommerce/conf/mime.types
PAYLOAD_DIR = $BASEDIR/$STORENAME/cgi-bin/payloads/

REDIRECT_URL =
DEBUG_LEVEL = 0

DEBUG_FILE  = $BASEDIR/$STORENAME/logs/CCMck.log

MSW_ACCEPTS = Visa,MasterCard,Discover,Amex
MSW_DUMMY_URL = https://$FQDN/html/eng/empty.wlt

TIMEOUT = 120
MERCHANT_EOF

    close MERCHANT;
    &release_lock("$STORENAME/conf/merchant_conf");


    ### ########################################################################
    ### Generate conf/database.conf. ###########################################
    ### ########################################################################
    &obtain_lock("$STORENAME/conf/database.conf");
    open DATABASE, ">$STORENAME/conf/database.conf";

    print DATABASE <<DATABASE_EOF;
site_datadriver  mysql
site_database_serv  localhost
site_dbiuser  $site_dbiuser
site_dbipwd  $site_dbipwd
site_database  $STORENAME
DATABASE_EOF

    close DATABASE;
    &release_lock("$STORENAME/conf/database.conf");


    ### ########################################################################
    ### Generate conf/machine.conf. ############################################
    ### ########################################################################

    ($mac_host, $mac_dom) = ($FQDN =~ m/^([^\.]+)(.*)$/);

    &obtain_lock("$STORENAME/conf/machine.conf");
    open MACHINE, ">$STORENAME/conf/machine.conf";

    print MACHINE <<MACHINE_EOF;
sitedomain  $mac_dom
siteserver  $mac_host
sitessldomain  $mac_dom
sitesslserver  $mac_host
ssl_prefix  https
mailserver  /usr/sbin/sendmail
replyto  $replyto
\$sr  /home/httpd
MACHINE_EOF

    close MACHINE;
    &release_lock("$STORENAME/conf/machine.conf");


    ### ########################################################################
    ### Generate conf/site.conf. ###############################################
    ### ########################################################################
    &obtain_lock("$STORENAME/conf/site.conf");
    open SITE, ">$STORENAME/conf/site.conf";

    print SITE <<SITE_EOF;
sitename  $sitename
sitecode  m
site_background
site_limit_jit  5
site_limit_all  5
maxtopnav  7
site_sku_totlen  8
site_sku_prelen  3
site_sku_sclen  1
site_sku_numlen  4

customer_service_email_address  $customer_service_email_address
order_email_address  $order_email_address
webmaster_email_address  $webmaster_email_address

interfaces  html
payprocess  $payprocess

hash_key  $hash_key

storestate  $storestate
taxrate  $taxrate

om_locale_lang_def  eng
om_locale_lang_act
om_locale_lang_inact
om_locale_curr_def  USD
om_locale_curr_act
currency_symbol  \$
SITE_EOF

    close SITE;
    &release_lock("$STORENAME/conf/site.conf");

    ### ########################################################################
    ### Put the 'generate_site.pl' script in crons. ############################
    ### ########################################################################
    open CRONS, ">/etc/cron.ac/$FQDN"; 
      print CRONS "#!/bin/sh\n\n";

      print CRONS "cd /home/httpd/$STORENAME\n";
      print CRONS "crons/generate_site.pl > /dev/null 2>&1\n";
    close CRONS;

    ### Make it read/execute only by root.
    chmod 0500, "/etc/cron.ac/$FQDN";


    ### ########################################################################
    ### Fix the CyberCash paths. ###############################################
    ### ########################################################################
    chdir "$BASEDIR/$STORENAME/cgi-bin/mck-cgi";
    system "for i in `ls`; do mv \$i \$i.old; sed -e 's/test-store/$STORENAME/' \$i.old > \$i; rm -f \$i.old; done;";


    ### ########################################################################
    ### - Generate the site ####################################################
    ### - Set final ownership/permissions. #####################################
    ### ########################################################################
    chdir "$BASEDIR/$STORENAME";
      system "crons/update_conf.pl";
      system "crons/generate_site.pl";

      system "chmod 775 transit transit/temp";
      # system "chmod 755 bits/eng/html/sel/47004-A.sel";
      # system "chmod 755 bits/eng/html/sel/74003-A.sel";
      system "chmod 775 bits/eng/html/sel/*";

      system "chmod 770 logs";

      system "chmod 755 cgi-bin/mck-cgi/*.cgi";
      system "chmod 755 cgi-bin/mck-cgi/MCK*";
      system "chmod 755 cgi-bin/mck-cgi/compute*";

      system "chmod 644 lib/*";
      system "chmod 750 tools/*";
      system "chmod 555 crons/*";
      system "chmod 660 conf/*";
      system "chmod 640 conf/site.pm conf/merchant_conf";



      system "chown root.webd transit transit/temp";
      # system "chown $STORE_USERNAME.webd bits/eng/html/sel/47004-A.sel";
      # system "chown $STORE_USERNAME.webd bits/eng/html/sel/74003-A.sel";

      system "chown root.webd logs";

      system "chown root.root cgi-bin/mck-cgi/*.cgi";
      system "chown root.root cgi-bin/mck-cgi/MCK*";
      system "chown root.root cgi-bin/mck-cgi/compute*";

      system "chown root.root lib/*";
      system "chown root.$STORE_GROUPNAME tools/* crons/*";
      system "chown root.webd conf/*";
      system "chown root.webd conf/site.pm conf/merchant_conf";



      system "chmod -R 775 bits images";
      system "chmod -R 550 tools";
      system "chown -R $STORE_USERNAME bits images";
      system "chgrp -R $STORE_GROUPNAME bits images";

      system "chown $STORE_USERNAME.webd bits/eng/html/sel/*";

      system "chgrp webd bits/eng/html/sel/47004-A.sel";
      system "chgrp webd bits/eng/html/sel/74003-A.sel";

      system "chgrp $STORE_GROUPNAME tools";


    ### ########################################################################
    ### Get the store up and running. ##########################################
    ### ########################################################################
    system "/etc/init.d/httpd stop";

      ### Reload LIDS config.
      &lids_toggle("off");
      &lids_reload();
      &lids_toggle("on");

    ### Clean and save the environment.
    my %ENV_RESTORE;
    %ENV_RESTORE  =  &clean_env();

      ### Restart the daemon.
      $out1 = `/etc/init.d/httpd start 2>&1`;

    ### Restore the environment.
    &restore_env(%ENV_RESTORE);



    ### ########################################################################
    ### I'm... sailing... away... </cartman> ###################################
    ### ########################################################################
    &redirect("/VHost");
  }

